-----------------------------------------------------------------------------------------------
-- Client Lua Script for Scrappy
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- Scrappy Module Definition
-----------------------------------------------------------------------------------------------
local Scrappy = {} 
 
-----------------------------------------------------------------------------------------------
-- Temporary Locals (Dabnabbit)
-----------------------------------------------------------------------------------------------
local updateDelay = 0.3
local lastRedrawUpdate = 0
local lastSalvageTimer = 0

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local knVersion = 5

local ktQualityCodeToBorderStrings = {
	[Item.CodeEnumItemQuality.Inferior]  = "BK3:UI_BK3_ItemQualityGrey",
	[Item.CodeEnumItemQuality.Average]   = "BK3:UI_BK3_ItemQualityWhite",
	[Item.CodeEnumItemQuality.Good]      = "BK3:UI_BK3_ItemQualityGreen",
	[Item.CodeEnumItemQuality.Excellent] = "BK3:UI_BK3_ItemQualityBlue",
	[Item.CodeEnumItemQuality.Superb]    = "BK3:UI_BK3_ItemQualityPurple",
	[Item.CodeEnumItemQuality.Legendary] = "BK3:UI_BK3_ItemQualityOrange",
	[Item.CodeEnumItemQuality.Artifact]  = "BK3:UI_BK3_ItemQualityMagenta"
}

local ktQualityCodeToColor = {
	[Item.CodeEnumItemQuality.Inferior]  = ApolloColor.new(0.6, 0.6, 0.6, 1.0),
	[Item.CodeEnumItemQuality.Average]   = ApolloColor.new(1.0, 1.0, 1.0, 0.9),
	[Item.CodeEnumItemQuality.Good]      = ApolloColor.new(0.0, 1.0, 0.0, 1.0),
	--[Item.CodeEnumItemQuality.Excellent] = ApolloColor.new(0.0, 0.0, 1.0, 1.0),
	[Item.CodeEnumItemQuality.Excellent] = ApolloColor.new(0.0, 0.5, 1.0, 1.0),
	[Item.CodeEnumItemQuality.Superb]    = ApolloColor.new(0.5, 0.0, 1.0, 1.0),
	[Item.CodeEnumItemQuality.Legendary] = ApolloColor.new(1.0, 0.5, 0.9, 1.0),
	[Item.CodeEnumItemQuality.Artifact]  = ApolloColor.new(1.0, 0.0, 0.6, 1.0)
}

local ktCodeEnumItemFamily = {}
ktCodeEnumItemFamily.Armor = 1
ktCodeEnumItemFamily.Weapon = 2
ktCodeEnumItemFamily.Gear = 7
ktCodeEnumItemFamily.Housing = 20
ktCodeEnumItemFamily.Costume = 26
ktCodeEnumItemFamily.Crafting = 27
ktCodeEnumItemFamily.Warplot = 30


local ktCodeEnumItemCategory = {}
-- ktCodeEnumItemFamily.Armor
ktCodeEnumItemCategory.LightArmor = 1
ktCodeEnumItemCategory.MediumArmor = 2
ktCodeEnumItemCategory.HeavyArmor = 3
-- ktCodeEnumItemFamily.Weapon
ktCodeEnumItemCategory.WeaponGreatsword = 8
ktCodeEnumItemCategory.WeaponRifle = 10 -- Not used
ktCodeEnumItemCategory.WeaponPaddles = 12
ktCodeEnumItemCategory.WeaponPistols = 16
ktCodeEnumItemCategory.WeaponStaff = 20 -- Not used
ktCodeEnumItemCategory.WeaponPsyblade = 22
ktCodeEnumItemCategory.WeaponClaws = 24
ktCodeEnumItemCategory.WeaponHeavyGun = 108
-- ktCodeEnumItemFamily.Gear
ktCodeEnumItemCategory.Gear = 7
-- ktCodeEnumItemFamily.Housing
ktCodeEnumItemCategory.Housing = 67
-- ktCodeEnumItemFamily.Costume
ktCodeEnumItemCategory.Costume = 99
-- ktCodeEnumItemFamily.Warplot
ktCodeEnumItemCategory.WarplotImprovment = 121
ktCodeEnumItemCategory.WarplotDeployableWeapon = 123
ktCodeEnumItemCategory.WarplotDeployableTrap = 124


-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Scrappy:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function Scrappy:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- Scrappy OnLoad
-----------------------------------------------------------------------------------------------
function Scrappy:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Scrappy.xml")
	self.xmlDoc = XmlDoc.CreateFromFile("Scrappy.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Scrappy OnDocLoaded
-----------------------------------------------------------------------------------------------
function Scrappy:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ScrappyForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.wndMain:SetSizingMinimum(300,300)
	    self.wndMain:Show(false, true)
	    if self.tOffsets then
	    	self.wndMain:SetAnchorOffsets(self.tOffsets.nLeft, self.tOffsets.nTop, self.tOffsets.nRight, self.tOffsets.nBottom)
	    end
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("sp", "OnScrappyOn", self)
		Apollo.RegisterSlashCommand("scrappy", "OnScrappyOn", self)

		Apollo.RegisterTimerHandler("ImprovedSalvageBulker", "RedrawSalvList", self)
		Apollo.CreateTimer("ImprovedSalvageBulker", updateDelay, false)
		Apollo.StopTimer("ImprovedSalvageBulker")
		-- //Dabnabbit: This ugly little guy will give a window for vacuum loot to land
		--	this allows a bunch of loot to be gathered without triggering immediately on the first item
		--	to avoids calling the inventory scan repeatedly for multiple loots!
		Apollo.RegisterEventHandler("UpdateInventory", "BulkLooter", self)
		Apollo.RegisterEventHandler("LootedItem", "SalvProductCheck", self)

		Apollo.RegisterEventHandler("RequestSalvageAll", "OnScrappyOn", self)

		Apollo.RegisterEventHandler("RequestSalvageAll", "AmaCominForYou", self)
		Apollo.RegisterTimerHandler("ImprovedSalvageAssasin", "StabStab", self)
		Apollo.CreateTimer("ImprovedSalvageAssasin", 0.01, false)
		Apollo.StopTimer()
		
		-- Do additional Addon initialization here
		self.wndSalvList = self.wndMain:FindChild("SalvList")
		self.tSalvList = {}
		self.bIsSalvHistory = false

		--//Dabnabbit: Allow persistence and load the saved log if available!
		if self.tSalvLog == nil then
			self.tSalvLog = {}
		end
		--//Dabnabbit: Allow persistence of window lock
		if self.bLocked == nil then
			self.bLocked = false
		elseif self.bLocked ~= false then
			self.bLocked = true --//Dabnabbit: Redundant error correction
		end
		self.wndMain:FindChild("Lock"):SetCheck(self.bLocked)
		self:SetWindowLock( self.wndMain, self.bLocked )

		-- filter options
		self.wndFilterOptions = Apollo.LoadForm(self.xmlDoc, "FilterOptions", self.wndMain, self)	
		self.wndFilterOptions:Show(false)

		-- keep ignored item list if only version changed
		local tItemsIgnored
		if self.tFilterOptions ~= nil and self.tFilterOptions.nVersion ~= knVersion then
			tItemsIgnored = self.tFilterOptions.tItemsIgnored
		end
		-- populate new filter option vars if do not exist or version change
		if self.tFilterOptions == nil or self.tFilterOptions.nVersion ~= knVersion then
			self.tFilterOptions = {}
			self.tFilterOptions.nVersion = knVersion
			-- by item quality
			self.tFilterOptions.tQualityFlags = {
				[Item.CodeEnumItemQuality.Inferior]  = false,
				[Item.CodeEnumItemQuality.Average]   = false,
				[Item.CodeEnumItemQuality.Good]      = false,
				[Item.CodeEnumItemQuality.Excellent] = false,
				[Item.CodeEnumItemQuality.Superb]    = false,
				[Item.CodeEnumItemQuality.Legendary] = false,
				[Item.CodeEnumItemQuality.Artifact]  = false,
			}
			-- by item family
			self.tFilterOptions.tFamilyFlags = {
				[ktCodeEnumItemFamily.Housing] = false,
				[ktCodeEnumItemFamily.Costume] = false,
				[ktCodeEnumItemFamily.Warplot] = false,
				[ktCodeEnumItemFamily.Weapon] = false
			}
			-- by item category
			self.tFilterOptions.tCategoryFlags = {
				[ktCodeEnumItemCategory.LightArmor] = false,
				[ktCodeEnumItemCategory.MediumArmor] = false,
				[ktCodeEnumItemCategory.HeavyArmor] = false,
				[ktCodeEnumItemCategory.Gear] = false
				--[ktCodeEnumItemCategory.FancyPants] = false
			}

			self.tFilterOptions.bFancyPants = false
			self.tFilterOptions.bBindOnEquip = false

			self.tFilterOptions.bMinReqLevel = false
			self.tFilterOptions.nMinReqLevel = 0			
			self.tFilterOptions.bMaxReqLevel = false
			self.tFilterOptions.nMaxReqLevel = 0

			self.tFilterOptions.bMinPowerLevel = false
			self.tFilterOptions.nMinPowerLevel = 0			
			self.tFilterOptions.bMaxPowerLevel = false
			self.tFilterOptions.nMaxPowerLevel = 0			

			-- by item id (table of item ids to bool)
			self.tFilterOptions.tItemsIgnored = {}
		end
		if tItemsIgnored then
			self.tFilterOptions.tItemsIgnored = tItemsIgnored
		end

		-- restore checkboxes
		local wndQuality = self.wndFilterOptions:FindChild("OptionContainer"):FindChild("Quality")
		wndQuality:FindChild("Inferior"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Inferior])
		wndQuality:FindChild("Average"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Average])
		wndQuality:FindChild("Good"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Good])
		wndQuality:FindChild("Excellent"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Excellent])
		wndQuality:FindChild("Superb"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Superb])
		wndQuality:FindChild("Legendary"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Legendary])
		wndQuality:FindChild("Artifact"):FindChild("Button"):SetCheck(self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Artifact])

		local wndEquipment = self.wndFilterOptions:FindChild("OptionContainer"):FindChild("Equipment")
		wndEquipment:FindChild("Light"):FindChild("Button"):SetCheck(self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.LightArmor])
		wndEquipment:FindChild("Medium"):FindChild("Button"):SetCheck(self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.MediumArmor])
		wndEquipment:FindChild("Heavy"):FindChild("Button"):SetCheck(self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.HeavyArmor])
		wndEquipment:FindChild("Gear"):FindChild("Button"):SetCheck(self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.Gear])
		wndEquipment:FindChild("Weapons"):FindChild("Button"):SetCheck(self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Weapon])
		wndEquipment:FindChild("MrFancyPants"):FindChild("Button"):SetCheck(self.tFilterOptions.bFancyPants)

		local wndMisc = self.wndFilterOptions:FindChild("OptionContainer"):FindChild("Misc")
		wndMisc:FindChild("Housing"):FindChild("Button"):SetCheck(self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Housing])
		wndMisc:FindChild("Costumes"):FindChild("Button"):SetCheck(self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Costume])
		wndMisc:FindChild("Warplots"):FindChild("Button"):SetCheck(self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Warplot])
		wndMisc:FindChild("BindOnEquip"):FindChild("Button"):SetCheck(self.tFilterOptions.bBindOnEquip)

		local wndLevel = self.wndFilterOptions:FindChild("OptionContainer"):FindChild("Level")
		-- min req
		wndLevel:FindChild("MinReqLevel"):FindChild("Button"):SetCheck(self.tFilterOptions.bMinReqLevel)
		wndLevel:FindChild("MinReqLevel"):FindChild("EditBox"):SetText(self.tFilterOptions.nMinReqLevel)
		-- max req
		wndLevel:FindChild("MaxReqLevel"):FindChild("Button"):SetCheck(self.tFilterOptions.bMaxReqLevel)
		wndLevel:FindChild("MaxReqLevel"):FindChild("EditBox"):SetText(self.tFilterOptions.nMaxReqLevel)
		-- min power
		wndLevel:FindChild("MinPowerLevel"):FindChild("Button"):SetCheck(self.tFilterOptions.bMinPowerLevel)
		wndLevel:FindChild("MinPowerLevel"):FindChild("EditBox"):SetText(self.tFilterOptions.nMinPowerLevel)
		-- max power
		wndLevel:FindChild("MaxPowerLevel"):FindChild("Button"):SetCheck(self.tFilterOptions.bMaxPowerLevel)
		wndLevel:FindChild("MaxPowerLevel"):FindChild("EditBox"):SetText(self.tFilterOptions.nMaxPowerLevel)
	end
end

-----------------------------------------------------------------------------------------------
-- Scrappy Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/scrappy"
function Scrappy:OnScrappyOn()
	if Apollo.FindWindowByName("ScrappyForm"):IsShown() then
		self:OnClose()
	else
		if GameLib.GetGameTime() - lastRedrawUpdate >= updateDelay then
			self.wndMain:Invoke() -- show the window
			self:RedrawSalvList()
		end
	end
end

function Scrappy:SalvProductCheck(tIt, tQuant)
	--Print("Logging: " .. tIt:GetName() .. " x " .. tQuant)
	if Apollo.FindWindowByName("ScrappyForm"):IsShown() then
		tTime = GameLib:GetGameTime()
		if tTime - lastSalvageTimer < 0.3 then --//Dabnabbit: This is a kind of silly hack to check if new items are from salvage
			local tTimeLcl = self:getLogTime()
			--Print("Salvage Logging: " .. tTimeLcl .. ", " .. tIt:GetName() .. " x " .. tQuant)
			table.insert(self.tSalvLog, {
				["tTime"] = tTimeLcl,
				["tItem"] = tIt:GetItemId(),
				["tItemQual"] = tIt:GetItemQuality(),
				["tQuantity"] = tQuant
			} )
		end
	end
end

function Scrappy:BulkLooter()
	if Apollo.FindWindowByName("ScrappyForm"):IsShown() then
		Apollo.StartTimer("ImprovedSalvageBulker")
	else
		Apollo.StopTimer("ImprovedSalvageBulker")
	end
end

function Scrappy:BuildToolTip(wnd, tItem, iIndex)
	wnd:SetTooltipDoc(nil)
	wnd:SetTooltipDocSecondary(nil)
	if tItem ~= nil then
		--Print("Tooltip: " .. tItem:GetItemId())
		if type(tItem) ~= "userdata" then
			Print("Error Tooltip tItem Type: " .. tItem)
			tItem = Item.GetDataFromId(tItem)
		end
		local tTooltipOpts = {}
		tTooltipOpts.bPrimary = true
		tTooltipOpts.itemModData = tItem.itemModData
		tTooltipOpts.strMaker = tItem.strMaker
		tTooltipOpts.arGlyphIds = tItem.arGlyphIds
		tTooltipOpts.tGlyphData = tItem.itemGlyphData
		if self.bIsSalvHistory == true and self.tSalvLog[iIndex].tTTOpts ~= nil then --//Dabnabbit: Try some tooltip recovery magic!
			--Print("Testing Magic")
			tTooltipOpts = self.tSalvLog[iIndex].tTTOpts
		end
		if tItem.itemModData ~= nil or tItem.strMaker ~= nil or tItem.arGlyphIds ~= nil or tItem.itemGlyphData ~= nil then
			Print("HOLY SHIT THERE IS EXTRA DATA?! @: " .. iIndex)
		end
		tTooltipOpts.itemCompare = tItem:GetEquippedItemForItemType()
		--Apollo.GetAddon("Rover"):AddWatch("TestTooltip Magic", tTooltipOpts, 0 )
		if Tooltip ~= nil and Tooltip.GetSpellTooltipForm ~= nil then
			Tooltip.GetItemTooltipForm(self, wnd, tItem, tTooltipOpts, tItem.nStackSize)
			return tTooltipOpts
		end
	end
end

--//Dabnabbit: Some attempted MouseOver script trickery
function Scrappy:onMouseEnter( wndHandler, wndControl, x, y )
	local wnd = wndHandler:GetParent()
	local wndName = wndHandler:GetName()
	local wndSalvTip = wnd:FindChild("SalvTip")
	if wndSalvTip == nil then return end
	if wndName == "SalvSelect" then
		wndSalvTip:SetSprite("commdisplay:sprComm_StaticComposite")	
		wndSalvTip:SetBGColor(ktQualityCodeToColor[2])
		wndSalvTip:SetText("")
	elseif wndName == "Icon" and self.bSalvTool == true then
		wndSalvTip:SetSprite("CRB_CharacterCreateSprites:sprCharC_GlowBackerMain")
		wndSalvTip:SetBGColor(ktQualityCodeToColor[4])
		wndSalvTip:SetTextColor(ktQualityCodeToColor[4])
		wndSalvTip:SetText(" Compare")
		--self:SetSalvTextColors(wndHandler, ktQualityCodeToColor[wnd:GetData().tQual])
	elseif wndName == "SalvClick" and self.bSalvTool == true then
		wndSalvTip:SetSprite("commdisplay:sprComm_StaticComposite")
		wndSalvTip:SetBGColor(ktQualityCodeToColor[2])
		wndSalvTip:SetTextColor(ApolloColor.new(0.8, 0.2, 0.0, 0.9))
		wndSalvTip:SetText(" Salvage?")
		self:SetSalvTextColors(wndHandler, ktQualityCodeToColor[wndControl:GetData().tQual])
		--wndSalvTip:SetText("")
	else
		Print("Scrappy: MouseOver on " .. wndName)
	end
	wndSalvTip:Show(true)
end

function Scrappy:onMouseExit( wndHandler, wndControl, x, y )
	local wnd = wndHandler:GetParent()
	local wndName = wndHandler:GetName()
	local wndSalvTip = wnd:FindChild("SalvTip")
	if wndSalvTip == nil then return end
	if wndName == "Icon" or wndName == "SalvClick" or wndName == "SalvSelect" then
		if self.bSalvTool == true then
			self:SetSalvTextColors(wndHandler, ktQualityCodeToColor[2])
		end
		wndSalvTip:Show(false)
	else
		Print("Scrappy: MouseExit on " .. wndName)
	end
end

--tTest:GetItemId()
--Item.GetDataFromId(tTest)
--//Dabnabbit: The core function for redrawing the salvage/log list
function Scrappy:RedrawSalvList(bForce)
	wndMain = Apollo.FindWindowByName("ScrappyForm")
	if wndMain:IsShown() then
		if self.bIsSalvHistory == true then --//Dabnabbit: Are we set to History Mode?
			--Print("In Salvage History!")
			self:DestroySalvList()
			self.tSalvList = {}
			local wndTag = wndMain:FindChild("Title2")
			wndTag:Show(true) --//Dabnabbit: Turn on the little Log title indicator
			wndTag:SetText("  Salvage Log")
			wndTag:SetTextColor(ApolloColor.new(0.0, 0.5, 1.0, 1.0))
			wndMain:FindChild("SalvButton"):Show(false) --//Dabnabbit: Turn off the salvage button
			wndMain:FindChild("ClearButton"):SetText("Clear Log") --//Dabnabbit: Change the ClearButton into log clear
			wndMain:FindChild("ClearButton"):SetTooltip("Clear Salvage Log")
			for x, tCode in ipairs(self.tSalvLog) do 
				tItem = Item.GetDataFromId(tCode.tItem)
				--Print(x .. " | Item: " .. tItem:GetName() .. " x " .. tCode.tQuantity)
				local wnd = Apollo.LoadForm(self.xmlDoc, "SalvItem", self.wndSalvList, self)
				local wndLogIcon = wnd:FindChild("LogIcon")
				wndLogIcon:Show(true)
				wnd:FindChild("SalvClick"):Show(false)
				local wndIcon = wnd:FindChild("Icon")
				wndIcon:SetSprite(tItem:GetIcon())
				wndIcon:SetData(tItem:GetItemId())
				--self:BuildToolTip(wndIcon, tItem, x)
				
				wndIcon:FindChild("BoE"):Show(false) --//Dabnabbit: No need for BOE check here right?
				local wndCount = wndIcon:FindChild("Count")
				local tQuantity = tCode.tQuantity
				if tQuantity < 1 then --//Dabnabbit: Check to see if the item was removed/added in the salvage window and quantity!
					wndLogIcon:SetSprite("CRB_Basekit:kitAccent_Frame_OrangeStrokePulse")
					wndCount:SetTextColor(ApolloColor.new(1.0, 0.0, 0.0, 0.75))
				else
					tQuantity = ("+"..tQuantity)
					wndLogIcon:SetSprite("CRB_Basekit:kitAccent_Frame_BlueStroke")
					wndCount:SetTextColor(ApolloColor.new(0.0, 1.0, 0.0, 0.75))
				end
				wndCount:SetText(tQuantity)
				wnd:FindChild("IgnoreButton"):Show(false) --//Dabnabbit: Ignore Button Irrelevant here... could repurpose? Nahh...
				local wndTextContainer = wnd:FindChild("TextContainer")
				wndTextContainer:FindChild("Name"):SetText(tItem:GetName())
				wndTextContainer:FindChild("Type"):SetText(tItem:GetItemTypeName())
				wndTextContainer:FindChild("Time"):SetText(tCode.tTime)
				local uSellPrice = tItem:GetSellPrice()
				if uSellPrice ~= nil then
					wnd:FindChild("SellValueCashWindow"):SetAmount(uSellPrice, 1)
				end
				--//Dabnabbit: This is just a quick crafted item quality color hack since they will always report QL as 2 (white)
				local tQuality = tCode.tItemQual
				--Print("tCode.tItemQual: " .. tQuality)
				if tQuality == nil then
					tQuality = tItem:GetItemQuality()
				end
				wnd:FindChild("QualityBorder"):SetSprite(ktQualityCodeToBorderStrings[tQuality])
				local wndSalvSelect = wnd:FindChild("SalvSelect")
				wndSalvSelect:SetBGColor(ktQualityCodeToColor[tItem:GetItemQuality()])
				local tData = {	["tId"] = tItem:GetItemId(),
								["tTTOpts"] = self:BuildToolTip(wndSalvSelect, tItem, x),
								["tQual"] = tItem:GetItemQuality() }
				wndSalvSelect:SetData(tData)
				
				--local wndSalvButton = wnd:FindChild("SalvButton")
				--wndSalvButton:Show(false)
				--wndSalvButton:SetData(tItem:GetItemId())
				--self:BuildToolTip(wndSalvButton, tItem, x)
				table.insert(self.tSalvList, wnd)
			end
			--self:sortTable()
			self.wndSalvList:ArrangeChildrenVert()
		else
			--Print("Not in Salvage History! So draw the normal salvage window")
			local wndTag = wndMain:FindChild("Title2")
			if self.bSalvTool ~= true then
				wndTag:Show(false)
				wndMain:FindChild("SalvButton"):Show(true) --//Dabnabbit: Turn on the salvage button
			else
				wndMain:FindChild("SalvButton"):Show(false) --//Dabnabbit: Turn off salvage button when in one-click mode				
				wndTag:Show(true)
				wndTag:SetText("  Salvage Tool")
				wndTag:SetTextColor(ApolloColor.new(1.0, 0.0, 0.0, 1.0))
			end
			wndMain:FindChild("ClearButton"):SetText("Clear Ignored") --//Dabnabbit: Change the ClearButton into log clear
			wndMain:FindChild("ClearButton"):SetTooltip("Clear Ignore List")
			local tPlayerUnit = GameLib.GetPlayerUnit()
			if (GameLib.GetGameTime() - lastRedrawUpdate >= updateDelay or bForce == true) and tPlayerUnit then
				--.GetLocalTime() returns (table) - as: nYear, nMonth, nDay, nDayOfWeek, nHour, nMinute, nSecond, strFormattedTime
				--local tTime = GameLib.GetLocalTime()
				--local tTime = ("Time.strFormattedTime: " .. tTime.nYear .. "/" .. tTime.nMonth .. "/" .. tTime.nDay .. "  " .. tTime.strFormattedTime)
				self:DestroySalvList()
				self.tSalvList = {}
				local tInvItems = tPlayerUnit:GetInventoryItems()
				if tInvItems then
					for i, tInventoryItem in ipairs(tInvItems) do
						if tInventoryItem then
							local tItem = tInventoryItem.itemInBag;
							-- FilterItem returns false if addon has been configured to hide item, returns true for items to be shown
							if tItem and self:FilterItem(tItem) then
								local wnd = Apollo.LoadForm(self.xmlDoc, "SalvItem", self.wndSalvList, self)
								local wndTextContainer = wnd:FindChild("TextContainer")
								wndTextContainer:FindChild("Name"):SetText(tItem:GetName())
								wndTextContainer:FindChild("Type"):SetText(tItem:GetItemTypeName())
								wndTextContainer:FindChild("Time"):Show(false)
								wnd:FindChild("LogIcon"):Show(false)
								local wndIcon = wnd:FindChild("Icon")
								wndIcon:SetSprite(tItem:GetIcon())
								--wndIcon:SetData(tItem:GetItemId())
								--Print("i: " .. i)
	
								local tDetailedInfo = tItem:GetDetailedInfo()
								if tDetailedInfo and tDetailedInfo.tPrimary and tDetailedInfo.tPrimary.tBind then
									wndIcon:FindChild("BoE"):Show(tDetailedInfo.tPrimary.tBind.bOnEquip and not tDetailedInfo.tPrimary.tBind.bSoulbound)
								end
								local nStackCount = tItem:GetStackCount()
								if nStackCount > 1 then
									wndIcon:FindChild("Count"):SetText(nStackCount)
								end
								wnd:FindChild("QualityBorder"):SetSprite(ktQualityCodeToBorderStrings[tItem:GetItemQuality()])

								local uSellPrice = tItem:GetSellPrice()
								if uSellPrice ~= nil then
									wnd:FindChild("SellValueCashWindow"):SetAmount(uSellPrice, 1)
								end
								
								wndSalvSelect = wnd:FindChild("SalvSelect")
								wndSalvSelect:SetBGColor(ktQualityCodeToColor[tItem:GetItemQuality()])
								local tData = {	["tId"] = tItem:GetItemId(),
												["tTTOpts"] = self:BuildToolTip(wndSalvSelect, tItem, x),
												["tQual"] = tItem:GetItemQuality(),
												["tInvId"] = tItem:GetInventoryId() }
								wndSalvSelect:SetData(tData)
								wndSalvSelect:SetBGColor(ktQualityCodeToColor[tItem:GetItemQuality()])
								wndSalvClick = wnd:FindChild("SalvClick")
								wndSalvClick:SetData(tData)
								if self.bSalvTool ~= true then
									wndSalvSelect:Show(true)
									wndSalvClick:Show(false)
									wndSalvClick:SetActionData()
									--wndSalvSelect:SetStyle("Overlapped", false )
									--wndSalvClick:SetStyle("IgnoreMouse", true )
								else
									--Print("Salv Tool Enabled")
									wndSalvSelect:Show(false)
									wndSalvClick:Show(true)
									wndSalvClick:SetActionData(GameLib.CodeEnumConfirmButtonType.SalvageItem, tData.tInvId)
									self:BuildToolTip(wndSalvClick, tItem, i)
									self:BuildToolTip(wndIcon, tItem, i)
									--wndSalvSelect:SetStyle("Overlapped", true )
									--wndSalvClick:SetStyle("IgnoreMouse", false )
								end
								local wndIgnore = wnd:FindChild("IgnoreButton")
								wndIgnore:SetData(tItem:GetItemId())
								wndIgnore:SetTooltip("Ignore " .. tItem:GetName())
								table.insert(self.tSalvList, wnd)
							end
						end
					end
					self.wndSalvList:ArrangeChildrenVert()
					if self.tSalvList[1] ~= nil and self.bSalvTool ~= true then
						self:SetSelect(1)
					else
						self:SetSalvTarget()
					end
				end
				lastRedrawUpdate = GameLib.GetGameTime()
			end
		end
	end
end

function Scrappy:sortTable(tOpt)
    tTable = {}
	--for n in pairs(self.tSalvList) do
	--	table.insert(tTable, self.tSalvList[n])
	--end
	for n in pairs(self.tSalvLog) do
		table.insert(tTable, self.tSalvLog[n])
	end
	Apollo.GetAddon("Rover"):AddWatch("SalvLog", self.tSalvLog)
	Apollo.GetAddon("Rover"):AddWatch("SortTest", tTable)
	--table.sort(tTable, function(a,b) return a<b end)
	--table.sort(tTable, function(a,b)
	--	local tItem1 = Item.GetDataFromId(a.tItem)
	--	tItem2 = Item.GetDataFromId(b.tItem)
	--	Print(tItem1:GetName() .. " < " .. tItem2:GetName())
	--	return tItem1:GetName() < tItem2:GetName()
		--return a<b 
	--end)
	--for i, n in ipairs(tTable) do
		--print(n)
	--end
end

-- Returns true (show item) or false (hide item) based on how the user has configured the addon
function Scrappy:FilterItem(tItem)
	if tItem == nil then return false end
	if not tItem:CanSalvage() then return false	end
	local tFilterOptions = self.tFilterOptions
	-- check ignore list
	if tFilterOptions.tItemsIgnored[tItem:GetItemId()] then return false end
	-- filter by quality
	if tFilterOptions.tQualityFlags[tItem:GetItemQuality()] then return false end
	-- filter by family 
	if tFilterOptions.tFamilyFlags[tItem:GetItemFamily()] then return false end
	-- filter by category
	if tFilterOptions.tCategoryFlags[tItem:GetItemCategory()] then	return false end
	-- filter unbound bind on equips
	if tFilterOptions.bBindOnEquip then
		local tDetailedInfo = tItem:GetDetailedInfo()
		if tDetailedInfo and tDetailedInfo.tPrimary and tDetailedInfo.tPrimary.tBind then
			if tDetailedInfo.tPrimary.tBind.bOnEquip and not tDetailedInfo.tPrimary.tBind.bSoulbound then
				return false
			end
		end
	end
	--//Dabnabbit: FancyPants filter addition!
	if tFilterOptions.bFancyPants and Apollo.GetAddonInfo("MrFancyPants").bRunning == 1 then
		local tFancyPantsTest = Apollo.GetAddon("MrFancyPants"):GetItemSetString(tItem)
		if tFancyPantsTest ~= nil then
			return false
		else
			return true
		end
	end
	-- filter by min req level
	if tFilterOptions.bMinReqLevel and tItem:GetRequiredLevel() < tFilterOptions.nMinReqLevel then return false end
	-- filter by max req level
	if tFilterOptions.bMaxReqLevel and tItem:GetRequiredLevel() > tFilterOptions.nMaxReqLevel then return false end
	-- filter by min item level
	if tFilterOptions.bMinPowerLevel and tItem:GetPowerLevel() < tFilterOptions.nMinPowerLevel then return false end
	-- filter by max item level
	if tFilterOptions.bMaxPowerLevel and tItem:GetPowerLevel() > tFilterOptions.nMaxPowerLevel then return false end
	return true
end

function Scrappy:getLogTime()
	local tTime = GameLib.GetLocalTime()
	return (tTime.nYear .. "/" .. tTime.nMonth .. "/" .. tTime.nDay .. "  " .. tTime.strFormattedTime)
end

function Scrappy:OnSalvItem( wndHandler, wndControl )
	--//NEED TO ADD A VERIFICATION THAT ITEM EXISTED
	lastSalvageTimer = GameLib.GetGameTime()
	local tData = wndControl:GetData()
	local tItem = tData.tId
	table.insert(self.tSalvLog, {
		["tTime"] = self:getLogTime(),
		["tItem"] = tItem,
		["tTTOpts"] = tData.tTTOpts,
		["tItemQual"] = tData.tQual,
		["tQuantity"] = -1
	} )
end

function Scrappy:IgnoreItem( wndHandler, wndControl, eMouseButton )
	local tData = wndControl:GetData()
	if type(tData) == "table" then
		tData = tData.tId
	end
	self.tFilterOptions.tItemsIgnored[tData] = true
	self:RedrawSalvList()
end

function Scrappy:DestroySalvList()
	if self.tSalvList == nil then
		return
	end
	for i, wnd in ipairs(self.tSalvList) do
		wnd:Destroy()
	end
	self.tSalvList = {}
end

function Scrappy:DestroySalvLog( wndHandler, wndControl, eMouseButton )
	self.tSalvLog = {}
	self:DestroySalvList()
	self:RedrawSalvList()
end

function Scrappy:ShowFilterOptions()
	self.wndFilterOptions:Show(true)
end

function Scrappy:HideFilterOptions()
	self.wndFilterOptions:Show(false)
end

function Scrappy:ShowSalvHistory( wndHandler, wndControl, eMouseButton )
	if eMouseButton == 0 and self.bSalvLogOn ~= false then
		self.bIsSalvHistory = true
		self:RedrawSalvList()
	elseif eMouseButton == 1 then
		self.bSalvLogOn = false
		--Apollo.GetAddon("Rover"):AddWatch("SalvLogButton", wndControl)
		--local tItemSlotBGPixie = {loc = {fPoints = {0,0,1,10},nOffsets = {0,0,0,0},},strSprite="WhiteFill", cr= "black",fRotation="0"}
		Print("Disable Log")
	end
end

function Scrappy:HideSalvHistory( wndHandler, wndControl, eMouseButton )
	if eMouseButton == 0 then
		self.bIsSalvHistory = false
		self:RedrawSalvList()
	elseif eMouseButton == 1 and self.bSalvLogOn ~= true then
		self.bSalvLogOn = true
		Print("Reenable Log")
	end
end

function Scrappy:EnableSalvTool( wndHandler, wndControl, eMouseButton )
	self.bSalvTool = true
	self:RedrawSalvList(true)
end

function Scrappy:DisableSalvTool( wndHandler, wndControl, eMouseButton )
	self.bSalvTool = false
	self:RedrawSalvList(true)
end

function Scrappy:SetSelect( iIdx )
	if iIdx ~= nil and self.tSalvList ~= nil then
		--Print("Index: " .. iIdx .. ", Name: " .. self.tSalvList[iIdx]:GetName())
		local wSelect = self.tSalvList[iIdx]:FindChild("SalvSelect")
		wSelect:SetCheck(true)
		self:onSalvSelect(wSelect, wSelect, 0)
	end
end

function Scrappy:onSalvSelect( wndHandler, wndControl, eMouseButton )
	--Print("SalvSelect " .. eMouseButton)
	local tData = wndControl:GetData()
	if eMouseButton == 1 and self.bIsSalvHistory ~= true then
		self:IgnoreItem(wndHandler, wndControl, eMouseButton )
	else
		wndControl:SetBGColor(ktQualityCodeToColor[2])
		self:SetSalvTextColors(wndControl, ktQualityCodeToColor[tData.tQual])
		self:SetSalvTarget(tData)
	end
end

function Scrappy:onSalvUnselect( wndHandler, wndControl, eMouseButton )
	--Print("SalvUnselect " .. eMouseButton)
	local tData = wndControl:GetData()
	if eMouseButton == 1 and self.bIsSalvHistory ~= true then
		self:IgnoreItem(wndHandler, wndControl, eMouseButton )
	else
		local wnd = wndHandler:GetParent()
		wndControl:SetBGColor(ktQualityCodeToColor[tData.tQual])
		--//NEED TO CLEAR OUT SALVAGE SELECTION!
		self:SetSalvTarget(nil)
		self:SetSalvTextColors(wndControl, ktQualityCodeToColor[2])
	end
end

function Scrappy:SetSalvTarget( tData )
	local wndMain = Apollo.FindWindowByName("ScrappyForm")
	if self.bSalvTool ~= true then
		wndSalvButton = wndMain:FindChild("SalvButton")
		if tData == nil then
			wndSalvButton:SetActionData(nil,nil)
			wndSalvButton:SetText("Nothing Selected")
			wndSalvButton:SetBGColor(ApolloColor.new(0.5, 0.5, 0.5, 0.5))
			--wndSalvButton:SetTextColor(ktQualityCodeToColor[1]) --//Dabnabbit: Can't set text color on action button?
		else
			wndSalvButton:SetActionData(GameLib.CodeEnumConfirmButtonType.SalvageItem, tData.tInvId)	
			wndSalvButton:SetText("Salvage Item")
			wndSalvButton:SetBGColor(ktQualityCodeToColor[2])
			--wndSalvButton:SetTextColor(ktQualityCodeToColor[2])
		end
		wndSalvButton:SetData(tData)
	else
		--Print("Beep")
	end
end

function Scrappy:SetSalvTextColors( wndCtrl, cClr )
	local wnd = wndCtrl:GetParent()
	local tData = wndCtrl:GetData()
	wndTextContainer = wnd:FindChild("TextContainer")
	wndTextContainer:FindChild("Name"):SetTextColor(cClr)
	wndTextContainer:FindChild("Type"):SetTextColor(cClr)
	wnd:FindChild("Icon"):FindChild("BoE"):SetTextColor(cClr)
end

function Scrappy:onClearButton( wndHandler, wndControl, eMouseButton )
	if wndControl:GetText() == "Clear Ignored" then
		self:ClearIgnoreList( wndHandler, wndControl, eMouseButton )
	elseif wndControl:GetText() == "Clear Log" then
		self:DestroySalvLog( wndHandler, wndControl, eMouseButton )
	else
		Print("Scrappy Error: Clear Type: " .. wndControl:GetText())
	end
end

function Scrappy:SetWindowLock( wnd, bOpt )
	if bOpt == nil then bOpt = self.bLocked end
	if wnd:GetName() == "ScrappyForm" then
	--Apollo.GetAddon("Rover"):AddWatch("ScrappyLock", wnd)
		local tStr = "Lock Window"
		if bOpt == true then tStr = "Unlock Window" end
    	wnd:SetStyle("Moveable", not bOpt )
		wnd:SetStyle("Sizable", not bOpt )
		self.bLocked = bOpt
		--if wnd:FindChild("Lock") then Apollo.GetAddon("Rover"):AddWatch("ScrappyLock",  wnd:FindChild("Lock")) end
		wnd:FindChild("Lock"):SetTooltip(tStr)
		wnd:FindChild("Corner"):Show(not bOpt)
	end
end

function Scrappy:WindowLock( wndHandler, wndControl, eMouseButton )
	wnd = wndControl:GetParent()
	self:SetWindowLock( wnd, true )
end

function Scrappy:WindowUnlock( wndHandler, wndControl, eMouseButton )
	wnd = wndControl:GetParent()
	self:SetWindowLock( wnd, false )
end

function Scrappy:OnClose()
	self.wndMain:Close()
	Apollo.StopTimer("ImprovedSalvageBulker")
end

function Scrappy:AmaCominForYou()
	Apollo.StartTimer("ImprovedSalvageAssasin")
end

function Scrappy:StabStab()
	local wndImpSalv = Apollo.FindWindowByName("ImprovedSalvageForm")
	if wndImpSalv then
		wndImpSalv:Show(false)
	end
end

---------------------------------------------------------------------------------------------------
-- FilterOptions Functions
---------------------------------------------------------------------------------------------------
-- Much uglies, to be refactored...

function Scrappy:HideInferior( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Inferior]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowInferior( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Inferior]  = false
	self:RedrawSalvList()
end

function Scrappy:HideAverage( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Average]   = true
	self:RedrawSalvList()
end

function Scrappy:ShowAverage( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Average]   = false
	self:RedrawSalvList()
end

function Scrappy:HideGood( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Good]      = true
	self:RedrawSalvList()
end

function Scrappy:ShowGood( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Good]      = false
	self:RedrawSalvList()
end

function Scrappy:HideExcellent( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Excellent] = true
	self:RedrawSalvList()
end

function Scrappy:ShowExcellent( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Excellent] = false
	self:RedrawSalvList()
end

function Scrappy:HideSuperb( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Superb]    = true
	self:RedrawSalvList()
end

function Scrappy:ShowSuperb( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Superb]    = false
	self:RedrawSalvList()
end

function Scrappy:HideLegendary( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Legendary] = true
	self:RedrawSalvList()
end

function Scrappy:ShowLegendary( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Legendary] = false
	self:RedrawSalvList()
end

function Scrappy:HideArtifact( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Artifact]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowArtifact( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tQualityFlags[Item.CodeEnumItemQuality.Artifact]  = false
	self:RedrawSalvList()
end

function Scrappy:HideLight( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.LightArmor]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowLight( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.LightArmor]  = false
	self:RedrawSalvList()
end

function Scrappy:HideMedium( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.MediumArmor]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowMedium( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.MediumArmor]  = false
	self:RedrawSalvList()
end

function Scrappy:HideHeavy( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.HeavyArmor]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowHeavy( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.HeavyArmor]  = false
	self:RedrawSalvList()
end

function Scrappy:HideGear( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.Gear]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowGear( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tCategoryFlags[ktCodeEnumItemCategory.Gear]  = false
	self:RedrawSalvList()
end

-- // Dabnabbit: Attempt to add FancyPants gear list filter
function Scrappy:HideFancyPants( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bFancyPants = true
	self:RedrawSalvList()
end

-- // Dabnabbit: Attempt to add FancyPants gear list filter
function Scrappy:ShowFancyPants( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bFancyPants = false
	self:RedrawSalvList()
end

function Scrappy:HideWeapons( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Weapon]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowWeapons( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Weapon]  = false
	self:RedrawSalvList()
end

function Scrappy:HideHousing( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Housing]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowHousing( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Housing]  = false
	self:RedrawSalvList()
end

function Scrappy:HideCostumes( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Costume]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowCostumes( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Costume]  = false
	self:RedrawSalvList()
end

function Scrappy:HideWarplots( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Warplot]  = true
	self:RedrawSalvList()
end

function Scrappy:ShowWarplots( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tFamilyFlags[ktCodeEnumItemFamily.Warplot]  = false
	self:RedrawSalvList()
end

function Scrappy:HideBindOnEquip( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bBindOnEquip = true
	self:RedrawSalvList()
end

function Scrappy:ShowBindOnEquip( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bBindOnEquip = false
	self:RedrawSalvList()
end

-- min req level 
function Scrappy:HideMinReqLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMinReqLevel = true
	self:RedrawSalvList()
end

function Scrappy:ShowMinReqLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMinReqLevel = false
	self:RedrawSalvList()
end

function Scrappy:MinReqLevelChanged(  wndHandler, wndControl, strText )
	local nNumber = tonumber(strText)
	if nNumber then
		self.tFilterOptions.nMinReqLevel = nNumber
	else
		wndControl:SetText(self.tFilterOptions.nMinReqLevel)
	end
	self:RedrawSalvList()
end

-- max req level
function Scrappy:HideMaxReqLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMaxReqLevel = true
	self:RedrawSalvList()
end

function Scrappy:ShowMaxReqLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMaxReqLevel = false
	self:RedrawSalvList()
end

function Scrappy:MaxReqLevelChanged(  wndHandler, wndControl, strText )
	local nNumber = tonumber(strText)
	if nNumber then
		self.tFilterOptions.nMaxReqLevel = nNumber
	else
		wndControl:SetText(self.tFilterOptions.nMaxReqLevel)
	end
	self:RedrawSalvList()
end

-- min power level
function Scrappy:HideMinPowerLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMinPowerLevel = true
	self:RedrawSalvList()
end

function Scrappy:ShowMinPowerLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMinPowerLevel = false
	self:RedrawSalvList()
end

function Scrappy:MinPowerLevelChanged(  wndHandler, wndControl, strText )
	local nNumber = tonumber(strText)
	if nNumber then
		self.tFilterOptions.nMinPowerLevel = nNumber
	else
		wndControl:SetText(self.tFilterOptions.nMinPowerLevel)
	end
	self:RedrawSalvList()
end

-- max power level
function Scrappy:HideMaxPowerLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMaxPowerLevel = true
	self:RedrawSalvList()
end

function Scrappy:ShowMaxPowerLevel( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.bMaxPowerLevel = false
	self:RedrawSalvList()
end

function Scrappy:MaxPowerLevelChanged(  wndHandler, wndControl, strText )
	local nNumber = tonumber(strText)
	if nNumber then
		self.tFilterOptions.nMaxPowerLevel = nNumber
	else
		wndControl:SetText(self.tFilterOptions.nMaxPowerLevel)
	end
	self:RedrawSalvList()
end

function Scrappy:ClearIgnoreList( wndHandler, wndControl, eMouseButton )
	self.tFilterOptions.tItemsIgnored = {}
	self:RedrawSalvList()
end

function Scrappy:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then return nil end
	self.tOffsets = {}
	self.tOffsets.nLeft, self.tOffsets.nTop, self.tOffsets.nRight, self.tOffsets.nBottom = self.wndMain:GetAnchorOffsets() 
	return {
		tOffsets = self.tOffsets,
		tFilterOptions = self.tFilterOptions,
		tSalvLog = self.tSalvLog,
		bLocked = self.bLocked
	}
end

function Scrappy:OnRestore(eLevel, tData)
	self.tFilterOptions = tData.tFilterOptions
	self.tOffsets = tData.tOffsets
	self.tSalvLog = tData.tSalvLog
	self.bLocked = tData.bLocked
end

-----------------------------------------------------------------------------------------------
-- Scrappy Instance
-----------------------------------------------------------------------------------------------
local ScrappyInst = Scrappy:new()
ScrappyInst:Init()
